from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.svm import SVC
from sklearn.tree import DecisionTreeClassifier

from .predict import Predict


class Predict_Classif(Predict):

    def __init__(self, data_train_path, dependent):
        Predict.__init__(self, data_train_path, dependent)
        self.svm = 'no value assigned yet'
        self.dtc = 'no value assigned yet'
        self.acc_svm = 'no value assigned yet'
        self.acc_dtc = 'no value assigned yet'

    """
    Prediction class for training of and inferencing with two types of classification models:
        Support Vector Machine (non-linear w/ polynomial kernel)
        Decision Tree Classifier

    Attributes:
        dependent (string): name of dependent variable (inherited from parent class Predict)
        data_train_path (string): path to training data set (inherited from parent class Predict)
        svm (Pipeline object): Pipeline object including StandardScaler and SVM classification
        dtc (model object): DecisionTreeClassifier model object
        acc_svm (float): accuracy of support vector machine
        acc_dtc (float): accuracy of decision tree classifer
    """

    def train_svm(self, degr, c, c0 = 0):
        """
        Function to: 
            (i) fit support vector machine model to training data
            (ii) calculate accuracy score on training data
        Args: 
			degr (float): degree of polynomial kernel function
            c (float): Regularization parameter (the smaller c the stronger the regularization)
            c0 (float): Independent term in kernel function
		Returns: 
			None
        https://scikit-learn.org/stable/modules/generated/sklearn.svm.SVC.html
        """        
        self.svm = Pipeline([
            ('scaler', StandardScaler()),
            ('svm_clf', SVC(kernel = 'poly',
                            degree = degr, 
                            coef0 = c0, 
                            C = c)
            )
        ])
        self.svm.fit(self.data_train_x, self.data_train_y)
        self.acc_svm = self.svm.score(self.data_train_x, self.data_train_y)

    def train_dtc(self, md):
        """
        Function to: 
            (i) fit decision tree classification model to training data
            (ii) calculate accuracy score on training data
        Args: 
			md (float): maximum depth of tree
		Returns: 
			None
        https://scikit-learn.org/stable/modules/generated/sklearn.tree.DecisionTreeClassifier.html?highlight=decisiontree#sklearn.tree.DecisionTreeClassifier
        """        
        self.dtc = DecisionTreeClassifier(max_depth = md)
        self.dtc.fit(self.data_train_x, self.data_train_y)
        self.acc_dtc = self.dtc.score(self.data_train_x, self.data_train_y)

    def predict_svm(self):
        """
        Function to: 
            predict outcome with fitted support vector machine model
        Args: 
			None
		Returns: 
			pred_svm (Numpy array): Array of predicted values
        """    
        pred_svm = self.svm.predict(self.data_pred)
        return pred_svm

    def predict_dtc(self):
        """
        Function to: 
            predict outcome with fitted decision tree classification model
        Args: 
			None
		Returns: 
			pred_dtc (Numpy array): Array of predicted values
        """    
        pred_dtc = self.dtc.predict(self.data_pred)
        return pred_dtc



# Import Predict class from predict module
#from .predict import Predict
# -> cannot import parent class, why?

# Import Predict_Classif class from predict_classif module
from .predict_classif import Predict_Classif
# Import Predict_Regr class from predict_regr module
from .predict_regr import Predict_Regr

#from .Gaussiandistribution import Gaussian
#from .Binomialdistribution import Binomial


import pandas as pd


class Predict:

    def __init__(self, data_train_path, dependent):
        self.dependent = dependent
        self.data_train_path = data_train_path
        self.data_train_x = 'no value assigned yet'
        self.data_train_y = 'no value assigned yet'
        self.data_pred_path = 'no value assigned yet'
        self.data_pred = 'no value assigned yet'

    """
    Generic Predict class for training of and inferencing with various predictive models
    Parent class of Predict_Classif and Predict_Regr

    Attributes:
        dependent (string): name of dependent variable
        data_train_path (string): path to training data set
        data_train_x (pandas DataFrame): n x m dimensional matrix of training dataset, excluding 
            dependent variable (n - # of training instances, m - # of features)
        data_train_y (Numpy array): n dimensional array of dependent variable
        data_pred_path (string): path to prediction data set
        data_pred (string): n x m dimensional matrix of feature values to predict dependent variable
    """
    
    def load_data_train(self):
        """
        Function to: 
            (i) load in training dataset from drive (accepted formats: xlsx, xls, csv)
            (ii) split into features dataset and array of dependent variable
        Args: 
			None
		Returns: 
			None
        """
        if self.data_train_path[-5:] == '.xlsx':
            self.data_train = pd.read_excel(self.data_train_path)
        elif self.data_train_path[-4:] == '.xls':
            self.data_train = pd.read_excel(self.data_train_path)
        elif self.data_train_path[-4:] == '.csv':
            self.data_train = pd.read_csv(self.data_train_path)
        else:
            print('[ERROR] Please provide data in format csv, xlsx, or xls.')
        self.data_train_x = self.data_train.loc[:,self.data_train.columns != self.dependent]
        self.data_train_y = self.data_train.loc[:,self.data_train.columns == self.dependent].values.flatten()

    def load_data_pred(self, data_pred_path):
        """
        Function to: 
            load in prediction dataset from drive (accepted formats: xlsx, xls, csv)
        Args: 
			data_pred_path
		Returns: 
			None
        """
        self.data_pred_path = data_pred_path
        if self.data_pred_path[-5:] == '.xlsx':
            self.data_pred = pd.read_excel(self.data_pred_path)
        elif self.data_pred_path[-4:] == '.xls':
            self.data_pred = pd.read_excel(self.data_pred_path)
        elif self.data_pred_path[-4:] == '.csv':
            self.data_pred = pd.read_csv(self.data_pred_path)
        else:
            print('[ERROR] Please provide data in format csv, xlsx, or xls.')


from sklearn.svm import SVR
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import PolynomialFeatures
from sklearn.linear_model import LinearRegression

from .predict import Predict


class Predict_Regr(Predict):

    def __init__(self, data_train_path, dependent):
        Predict.__init__(self, data_train_path, dependent)
        self.svmr = 'no value assigned yet'
        self.lreg = 'no value assigned yet'
        self.acc_svmr = 'no value assigned yet'
        self.acc_lreg = 'no value assigned yet'

    """
    Prediction class for training and inferencing two types of regression models:
        Support Vector Machine Regression (non-linear w/ polynomial kernel)
        Linear Regression w/ polynomial features

    Attributes:
        dependent (string): name of dependent variable (inherited from parent class Predict)
        data_train_path (string): path to training data set (inherited from parent class Predict)
        svmr (model object): Support vector machine regression model object
        lreg (model object): LinearRegression model object (fit with OLS)
        acc_svmr (float): coefficient of determination (R2) of support vector machine regression
        acc_lreg (float): coefficient of determination (R2) of linear regression
    """

    def train_svmr(self, degr, c):
        """
        Function to: 
            (i) fit support vector machine regression model to training data
            (ii) calculate coefficient of determination (R2) of SVM regression on training data
        Args: 
			degr (float): degree of polynomial kernel function
            c (float): Regularization parameter (the smaller c the stronger the regularization)
		Returns: 
			None
        https://scikit-learn.org/stable/modules/generated/sklearn.svm.SVR.html#sklearn.svm.SVR
        """     
        self.svmr = SVR(kernel = "poly", 
                       degree = degr, 
                       C = c, 
                       epsilon = 0.1
                       )
        self.svmr.fit(self.data_train_x, self.data_train_y)
        self.acc_svmr = self.svmr.score(self.data_train_x, self.data_train_y)

    def train_lreg(self, degr, bias):
        """
        Function to: 
            (i) fit linear regression model (w/ polynomial features) to training dataw with OLS
            (ii) calculate coefficient of determination (R2) of linear regression model on training data
        Args: 
			degr (float): degree of polynomial
            bias (Boolean): Include intercept / bias term (yes or no)
		Returns: 
			None
        https://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LinearRegression.html
        https://scikit-learn.org/stable/modules/generated/sklearn.preprocessing.PolynomialFeatures.html
        """
        self.lreg = Pipeline([
            ("poly_features", PolynomialFeatures(degree = degr, include_bias = bias)),
            ("lin_reg", LinearRegression()),
            ])
        self.lreg.fit(self.data_train_x, self.data_train_y)
        self.acc_lreg = self.lreg.score(self.data_train_x, self.data_train_y)

    def predict_svmr(self):
        """
        Function to: 
            predict outcome with fitted support vector machine regression model
        Args: 
			None
		Returns: 
			pred_svmr (Numpy array): Array of predicted values
        """
        pred_svmr = self.svmr.predict(self.data_pred)
        return pred_svmr

    def predict_lreg(self):
        """
        Function to: 
            predict outcome with linear regression model
        Args: 
			None
		Returns: 
			pred_lreg (Numpy array): Array of predicted values
        """
        pred_lreg = self.lreg.predict(self.data_pred)
        return pred_lreg


from setuptools import setup

setup(name='predictions_210406',
      version='0.1',
      description='Prediction models for classification and regression',
      packages=['predictions_210406'],
      author = 'John Bunge',
      author_email = 'myemail@somethingsomething.com',
      zip_safe=False)
